#NoTrayIcon
#include "Aut2Exe\Include\ButtonConstants.au3"
#include "Aut2Exe\Include\ComboConstants.au3"
#include "Aut2Exe\Include\GUIConstantsEx.au3"
#include "Aut2Exe\Include\WindowsConstants.au3"
#include "Aut2Exe\Include\GuiComboBox.au3"
#include "Aut2Exe\Include\EditConstants.au3"
#include "Aut2Exe\Include\GuiComboBoxEx.au3"

; include our re-branding info and other global variables
#include 'InstantSupportDefs.au3'


;***********************
;  Set Script Options  *
;*******************************************************************************
Opt('GUIOnEventMode', 1)     ; Enable our event handlers


;**************
;  Functions  *
;*******************************************************************************

; Add $Id to head of ID List, avoid duplicate entries
Func AddToList($Id)
	; Remove $Id from list if already present and not at head of list
	$IDList = StringReplace($IDList, '|' & $Id , '')

	; Don't save more than ListMax, keep in mind we assume a 6 digit number.
	If StringLen($IDList) >= ($IDListMax * 6 + ($IDListMax - 1)) Then
		; Maximum ID's in list, trim
		If StringInStr(StringTrimRight($IDList, 7), $Id) = 0 Then $IDList = $Id & "|" & StringTrimRight($IDList, 7)
	Else
		; Maximum ID's not yet reached.
		If $IDList = "" Then
			$IDList = $Id
		Else
			If StringInStr($IDList, $Id) = 0 Then $IDList = $Id & "|" & $IDList
		EndIf
	EndIf
EndFunc


;*******************
;  Event Handlers  *
;*******************************************************************************

Func doGuiClose()
	Exit
EndFunc

Func doButtonClicked()
	If $EncryptViewer Then
		; Start Viewer with encryption for Windows
		ShellExecute(@ScriptDir & "\Bin\vncviewer.exe", "-proxy " & $RepeaterAddress & " ID:" & $IDNumber & " -quickoption " & $Quality & " -keepalive 1 -dsmplugin SecureVNCPlugin.dsm")
	Else
		; Start Viewer without encryption for Mac
		ShellExecute(@ScriptDir & "\Bin\vncviewer.exe", "-proxy " & $RepeaterAddress & " ID:" & $IDNumber & " -quickoption " & $Quality & " -keepalive 1")
	EndIf

	AddToList($IDNumber)

	; Save IDList in chunkviewer.ini
	IniWrite(@ScriptDir & "\Bin\chunkviewer.ini", "ChunkViewer", "List", $IDList)

	Exit
EndFunc

Func doClearHistory()
	GUICtrlSetData($StatusLabel, "History Cleared.")
	GUICtrlSetData($IdListCombo, "")
	IniWrite(@ScriptDir & "\Bin\chunkviewer.ini", "ChunkViewer", "List", "")
EndFunc

Func doAbout()
	GUICtrlSetData($StatusLabel, $BrandName & " Remote Viewer.")
	$StatusCounter = 30  ; Display About message for 30 x 100 msecs = 3 secs
EndFunc


;***********************************
;  Global Variables and Constants  *
;*******************************************************************************

Global $GUI_ENABLE_DEFBUTTON = 576
Global $IDNumber = ""

; Read settings file.
Global $RepeaterAddress = IniRead(@ScriptDir & "\Bin\chunkviewer.ini", "Repeater", "Address", "")
Global $IDList = IniRead(@ScriptDir & "\Bin\chunkviewer.ini", "ChunkViewer", "List", "")
Global $IDListMax = IniRead(@ScriptDir & "\Bin\chunkviewer.ini", "ChunkViewer", "ListMax", "")
Global $Quality = IniRead(@ScriptDir & "\Bin\chunkviewer.ini", "ChunkViewer", "Quality", "")

Global $StatusCounter = 0  ; Timer used to reset Status message after 3 secs


;***************
;  Create GUI  *
;*******************************************************************************

Global $ViewerForm = GUICreate($BrandName & " Remote Viewer", 300, 80, -1, -1)
GUISetBkColor(0xFFFFFF)
Global $ConnectButton = GUICtrlCreateButton("Connect", 185, 10, 100, 39)
GUICtrlSetFont(-1, 12, 800, 0, "MS Sans Serif")
GUICtrlSetState($ConnectButton, $GUI_DISABLE)
Global $IdListCombo = GUICtrlCreateCombo("", 15, 11, 155, 25)
GUICtrlSetFont(-1, 20, 800, 0, "MS Sans Serif")
;_GUICtrlComboBox_LimitText($IdListCombo, 6)	; Limit the number of characters to 6 for input.
Global $Line1 = GUICtrlCreateLabel("", 0, 60, 300, 1)
GUICtrlSetBkColor($Line1, 0x000000)
Global $StatusLabel = GUICtrlCreateLabel("", 5, 65, 300, 15)
GUICtrlSetFont($StatusLabel, 8, 400, 0, "MS Sans Serif")


; Create right-click context menu for Form
Global $ContextMenu = GUICtrlCreateContextMenu()
Global $ContextMenuItemHistory = GUICtrlCreateMenuItem("Clear History", $ContextMenu)
Global $ContextMenuSeparator1 = GUICtrlCreateMenuItem("", $ContextMenu)
Global $ContextMenuItemAbout = GUICtrlCreateMenuItem("About", $ContextMenu)

; Fill Combo1 and show current repeater address.
If $ConnectIdType = $ConnectIdFixed And $ConnectId <> 0 Then
	AddToList($ConnectId)
EndIf

GUICtrlSetData($IdListCombo, $IDList)
GUISetState(@SW_SHOW, $ViewerForm)
GUICtrlSetData($StatusLabel, "Using: " & $RepeaterAddress)

; Select first ID in Combo Box's list
_GUICtrlComboBox_SetCurSel($IdListCombo, 0)


;**************************
;  Assign Event Handlers  *
;*******************************************************************************

GUISetOnEvent($GUI_EVENT_CLOSE, 'doGuiClose')
GUICtrlSetOnEvent($ConnectButton, 'doButtonClicked')
GUICtrlSetOnEvent($ContextMenuItemHistory, 'doClearHistory')
GUICtrlSetOnEvent($ContextMenuItemAbout, 'doAbout')


;*****************************************
;  Main Application and Background Loop  *
;*******************************************************************************

While True
	; Disable the Connect button if empty, alphabetic characters or incorrect ID format.
	If $IDNumber <> GUICtrlRead($IdListCombo) Then
		$IDNumber = GUICtrlRead($IdListCombo)
;		If $IDNumber <> "" And StringIsDigit($IDNumber) = 1 And $IDNumber >= 100000 And $IDNumber <= 999999 Then
		If $IDNumber <> "" And StringIsDigit($IDNumber) = 1 Then
			GUICtrlSetState($ConnectButton, $GUI_ENABLE_DEFBUTTON)
		Else
			GUICtrlSetState($ConnectButton, $GUI_DISABLE)
		EndIf
	EndIf

	; Reset Status message after timeout
	If $StatusCounter > 0 Then
		$StatusCounter = $StatusCounter - 1
		If $StatusCounter = 0 Then GUICtrlSetData($StatusLabel, "Using: " & $RepeaterAddress)
	EndIf

	Sleep(100)
WEnd
